#!/usr/bin/env python
import fileinput
import csv
import sys
import re

# This prevents prematurely closed pipes from raising
# an exception in Python
from signal import signal, SIGPIPE, SIG_DFL
signal(SIGPIPE, SIG_DFL)

def is_insert(line):
    """
    Returns true if the line begins a SQL insert statement.
    """
    return line.startswith('INSERT INTO') or False


def get_values(line):
    """
    Returns the portion of an INSERT statement containing values
    """
    return line.partition('` VALUES ')[2].strip()


def values_sanity_check(values):
    """
    Ensures that values from the INSERT statement meet basic checks.
    """
    assert values
    assert values[0] == '('
    assert values[-2:] == ');'
    # Assertions have not been raised
    return True

def clean_quotes_for_fread(col):
    col = re.sub('\t', ' ', col)
    if col.startswith('"'):
        return('"' + col + '"')
    else:
        return(col)

def parse_values(values, outfile):
    """
    Given a file handle and the raw values from a MySQL INSERT
    statement, write the equivalent CSV to the file
    """
    values = values.rstrip(");")
    values = values.lstrip("(")

    reader = csv.reader(values.split("),("), delimiter=',',
                        doublequote=False,
                        escapechar='\\',
                        quotechar="'",
                        strict=True)

    for reader_row in reader:
        print("\t".join([clean_quotes_for_fread(col) for col in reader_row]))

def main():
    """
    Parse arguments and start the program
    """
    # Iterate over all lines in all files
    # listed in sys.argv[1:]
    # or stdin if no args given.
    try:
        for line in fileinput.input():
            # Look for an INSERT statement and parse it.
            if is_insert(line):
                values = get_values(line.strip().rstrip())
                if values_sanity_check(values):
                    parse_values(values, sys.stdout)
    except KeyboardInterrupt:
        sys.exit(0)

if __name__ == "__main__":
    main()
